/*******************************************************************************
* Gtk-FMRadio: This is free software, distributed under the Gnu GPL.           *
* 2000/02/05 (C) Fernando J. G. Pereira. (fjp@linuxstart.com)                  *
*******************************************************************************/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <linux/videodev.h>

/* Use the script "/dev/MAKEDEV to create device files: */
#define DEVICE "/dev/radio" 
#define RESOURCE_FILE ".gtk-fmradio" 
#define MAX_STATIONS 20


typedef struct {
    char name[20];
    float freq;
} station;



int device_fd = -1;
int n_stations = 0;
station stations[MAX_STATIONS];
gboolean muted = FALSE;
GtkWidget* about_win = 0;
GtkAdjustment* freq_adj = 0;
GtkCombo* station_list = 0;
GtkStatusbar* statusbar = 0;
GtkToggleButton* mute_toggle = 0;


int search_station( char* name )
{
    int i;

    for( i = 0; i < n_stations; ++i ) {
	if( strcmp( name, stations[i].name ) == 0 ) {
	    gtk_adjustment_set_value( freq_adj, stations[i].freq );
	    return i;
	}
    }
    return -1;
}


void fill_station_list()
{
    int i;
    GList *list = NULL;

    for( i = 0; i < n_stations; ++i ) 
	list = g_list_append( list, stations[i].name );

    if( list == NULL ) list = g_list_append( list, "<EMPTY>" );

    gtk_combo_set_popdown_strings( station_list, list );
}


void check_muted()
{
    if( muted ) {
        if( mute_toggle ) gtk_toggle_button_set_active( mute_toggle, FALSE );
        muted = FALSE;
    }
}


void
radio_setfreq(int fd, float freq)
{
    unsigned long ifreq = (unsigned long)(16000 * freq);
    if( ioctl(fd, VIDIOCSFREQ, &ifreq) < 0 ) perror( "Set Freq" );
}



float
radio_getfreq(int fd )
{
    unsigned long ifreq = 0;
    if( ioctl(fd, VIDIOCGFREQ, &ifreq) < 0 ) perror( "Get Freq" );
    return (float)ifreq / 16000;
}


void
radio_setvol(int fd, float vol)
{
    struct video_audio audio;

    if( ioctl(fd, VIDIOCGAUDIO, &audio) < 0 ) perror( "Get Audio" );
    audio.volume = (int)(0.5 + vol * 65535 / 10);
    if( ioctl(fd, VIDIOCSAUDIO, &audio) < 0 ) perror( "Set Audio" );
    check_muted();
}



float
radio_getvol( int fd )
{
    struct video_audio audio;

    if( ioctl(fd, VIDIOCGAUDIO, &audio) < 0 ) perror( "Get Audio" );
    return audio.volume * 10.0 / 65535;
}



char*
radio_getname( int fd )
{
    static char name[32];
    struct video_audio audio;

    if( ioctl(fd, VIDIOCGAUDIO, &audio) < 0 ) perror( "Get Audio" );
    strncpy( name, audio.name, 32 );
    return name;
}


void
radio_mute( int fd, int mute )
{
    struct video_audio audio;

    if( ioctl(fd, VIDIOCGAUDIO, &audio) < 0 ) perror( "Get Audio" );
    audio.flags &= ~VIDEO_AUDIO_VOLUME;
    if( mute ) {
        audio.flags |= VIDEO_AUDIO_MUTE;
	muted = TRUE;
    }
    if( ioctl(fd, VIDIOCSAUDIO, &audio) < 0 ) perror( "Set Audio" );
}



void
radio_stereo( int fd, int stereo )
{
    struct video_audio audio;

    if( ioctl(fd, VIDIOCGAUDIO, &audio) < 0 ) perror( "Get Audio" );
    if( stereo ) audio.mode = VIDEO_SOUND_STEREO;
    else audio.mode = VIDEO_SOUND_MONO;
    if( ioctl(fd, VIDIOCSAUDIO, &audio) < 0 ) perror( "Set Audio" );
    check_muted();
}



void
on_quit_clicked                        (GtkButton       *button,
                                        gpointer         user_data)
{
    close( device_fd );
    gtk_main_quit();
}



void
on_st_name_changed                     (GtkEditable     *editable,
                                        gpointer         user_data)
{
    int st;
    char* name = gtk_editable_get_chars( editable, 0, -1 );

    st = search_station( name );
    if( st >= 0 ) gtk_adjustment_set_value( freq_adj, stations[st].freq );
}



void
on_freq_changed( GtkAdjustment* adj, GtkWidget wid )
{
    radio_setfreq( device_fd, adj->value );
}


void
on_freq_map                            (GtkWidget       *widget,
                                        gpointer         user_data)
{
    freq_adj = gtk_range_get_adjustment( GTK_RANGE(widget) );
    gtk_signal_connect( GTK_OBJECT( freq_adj ), "value_changed", 
			GTK_SIGNAL_FUNC( on_freq_changed ), widget );
    gtk_adjustment_set_value( freq_adj,  radio_getfreq( device_fd ) );
}


void
on_vol_changed( GtkAdjustment* adj, GtkWidget wid )
{
    radio_setvol( device_fd, adj->value );
}


void
on_vol_map                             (GtkWidget       *widget,
                                        gpointer         user_data)
{
    GtkAdjustment* adj = gtk_range_get_adjustment( (GtkRange*)widget );
    gtk_signal_connect( GTK_OBJECT( adj ), "value_changed", 
			GTK_SIGNAL_FUNC( on_vol_changed ), widget );
    gtk_adjustment_set_value( adj, radio_getvol( device_fd ) );
}


void
on_mainwin_realize                     (GtkWidget       *widget,
                                        gpointer         user_data)
{
    char line[80];
    FILE* fptr;
    char* device;
    char resource_file[1024];
    
    device = getenv( "RADIO_DEVICE" );
    if( device == NULL ) device = DEVICE;

    if ((device_fd = open(device, O_RDONLY))==-1) {
	perror( device );
	exit(1);
    }
    radio_mute( device_fd, FALSE );
    radio_stereo( device_fd, TRUE );

    n_stations = 0;

    sprintf( resource_file, "%s/%s", getenv( "HOME" ), RESOURCE_FILE );
    fptr = fopen( resource_file, "r" );
    if( fptr == NULL ) {
        perror( resource_file );
        return;
    }

    while( n_stations < MAX_STATIONS && !feof( fptr ) ) {
        if( fgets( line, 80, fptr ) == NULL ) break;
        if( sscanf( line, "%[^\t] %f", stations[n_stations].name,
		    &stations[n_stations].freq ) == 2 ) ++n_stations;
    }
    fclose( fptr );
}


void
on_mute_toggled                        (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
     radio_mute( device_fd, gtk_toggle_button_get_active( togglebutton ) );
     mute_toggle = togglebutton;
}


void
on_stereo_toggled                      (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
     radio_stereo( device_fd, gtk_toggle_button_get_active( togglebutton ) );
}


gboolean
on_mainwin_delete_event                (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  gtk_main_quit();
  return FALSE;
}


void
on_station_map                         (GtkWidget       *widget,
                                        gpointer         user_data)
{
    station_list = GTK_COMBO( widget );
    fill_station_list();
}


void
on_statusbar_map                      (GtkWidget       *widget,
                                        gpointer         user_data)
{
    int ctx;

    statusbar = GTK_STATUSBAR( widget );
    ctx = gtk_statusbar_get_context_id( statusbar, "RADIO" );
    gtk_statusbar_push( statusbar, ctx, radio_getname( device_fd ) );
}



void
on_add_clicked                         (GtkButton       *button,
                                        gpointer         user_data)
{
    float freq = radio_getfreq( device_fd );
    char* name = gtk_entry_get_text( GTK_ENTRY( station_list->entry ) );
    int st = search_station( name );

    if( st < 0 ) {
	if( n_stations >= MAX_STATIONS ) return;
        st = n_stations++;
	strncpy( stations[st].name, name, 20 );
	fill_station_list();
    }
    stations[st].freq = freq;
    gtk_entry_set_text( GTK_ENTRY( station_list->entry ),
			stations[st].name );
}


void
on_del_clicked                         (GtkButton       *button,
                                        gpointer         user_data)
{
    char* name = gtk_entry_get_text( GTK_ENTRY( station_list->entry ) );
    int st = search_station( name );
    if( st >= 0 )
        while( ++st < n_stations ) stations[st-1] = stations[st];
    --n_stations;
    fill_station_list();
}


void
on_save_clicked                        (GtkButton       *button,
                                        gpointer         user_data)
{
    int i;
    FILE* fptr;

    char resource_file[1024];
    sprintf( resource_file, "%s/%s", getenv( "HOME" ), RESOURCE_FILE );
    
    fptr = fopen( resource_file, "w" );
    if( fptr == NULL ) {
        perror( resource_file );
        return;
    }

    for( i = 0; i < n_stations; ++i )
        fprintf( fptr, "%s\t\t%g\n", stations[i].name, stations[i].freq );
    fclose( fptr );
}


void
on_about_clicked                       (GtkButton       *button,
                                        gpointer         user_data)
{
    about_win = create_about_win();
    gtk_widget_show( about_win );
}


void
on_ok_button_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
    gtk_widget_destroy( about_win );
    about_win = 0;
}

